@@ -84,6 +84,18 @@ class DataSetCache(Generic[DatasetType]):
 
         return self._data
 
+    def prepare(self) -> None:
+        """
+        Set up the internal datastructure of the cache.
+        Must be called after the dataset has been setup with
+        interdependencies but before data is added to the dataset.
+        """
+
+        if self._data == {}:
+            self._data = self.rundescriber.interdeps._empty_data_dict()
+        else:
+            raise RuntimeError("Cannot prepare a cache that is not empty")
+
     def load_data_from_db(self) -> None:
         """
         Load the data from an on-disk format in case the cache is not live
@@ -299,7 +311,15 @@ def _merge_data(existing_data: Mapping[str, np.ndarray],
                 ) -> Tuple[Dict[str, np.ndarray], Optional[int]]:
 
     subtree_merged_data = {}
-    subtree_parameters = set(existing_data.keys()) | set(new_data.keys())
+    subtree_parameters = existing_data.keys()
+
+    if not set(new_data.keys()).issubset(set(existing_data.keys())):
+        raise RuntimeError(
+            "Trying to add unexpected key to cache."
+            "The following keys were unexpected: "
+            f"{set(new_data.keys() - existing_data.keys())}"
+        )
+
     new_write_status: Optional[int]
     single_param_merged_data, new_write_status = _merge_data_single_param(
         existing_data.get(meas_parameter),
@@ -331,13 +351,11 @@ def _merge_data_single_param(
         shape: Optional[Tuple[int, ...]],
         single_tree_write_status: Optional[int]) -> Tuple[Optional[np.ndarray], Optional[int]]:
     merged_data: Optional[np.ndarray]
-    if existing_values is not None and new_values is not None:
-        (merged_data,
-         new_write_status) = _insert_into_data_dict(
-            existing_values,
-            new_values,
-            single_tree_write_status,
-            shape=shape
+    if (
+        existing_values is not None and existing_values.size != 0
+    ) and new_values is not None:
+        (merged_data, new_write_status) = _insert_into_data_dict(
+            existing_values, new_values, single_tree_write_status, shape=shape
         )
     elif new_values is not None:
         (merged_data,
@@ -359,7 +377,7 @@ def _create_new_data_dict(new_values: np.ndarray,
                           ) -> Tuple[np.ndarray, int]:
     if shape is None:
         return new_values, new_values.size
-    else:
+    elif new_values.size > 0:
         n_values = new_values.size
         data = np.zeros(shape, dtype=new_values.dtype)
 
@@ -370,6 +388,8 @@ def _create_new_data_dict(new_values: np.ndarray,
 
         data.ravel()[0:n_values] = new_values.ravel()
         return data, n_values
+    else:
+        return new_values, new_values.size
 
 
 def _insert_into_data_dict(
@@ -378,6 +398,9 @@ def _insert_into_data_dict(
         write_status: Optional[int],
         shape: Optional[Tuple[int, ...]]
 ) -> Tuple[np.ndarray, Optional[int]]:
+    if new_values.size == 0:
+        return existing_values, write_status
+
     if shape is None or write_status is None:
         try:
             data = np.append(existing_values, new_values, axis=0)
@@ -461,7 +484,8 @@ class DataSetCacheWithDBBackend(DataSetCache["DataSet"]):
         self._dataset.completed = completed(self._dataset.conn, self._dataset.run_id)
         if self._dataset.completed:
             self._loaded_from_completed_ds = True
-
+        if self._data == {}:
+            self.prepare()
         (
             self._write_status,
             self._read_status,
@@ -474,5 +498,8 @@ class DataSetCacheWithDBBackend(DataSetCache["DataSet"]):
             self._read_status,
             self._data,
         )
-        if not all(status is None for status in self._write_status.values()):
+        data_not_read = all(
+            status is None or status == 0 for status in self._write_status.values()
+        )
+        if not data_not_read:
             self._live = False
