@@ -4,12 +4,13 @@ Test suite for monitor
 import asyncio
 import json
 import random
-import websockets
 
 import pytest
-from qcodes.monitor import monitor
+import websockets
+
 from qcodes.instrument.base import Parameter
-from qcodes.tests.instrument_mocks import DummyInstrument
+from qcodes.monitor import monitor
+from qcodes.tests.instrument_mocks import DummyChannelInstrument, DummyInstrument
 
 monitor.WEBSOCKET_PORT = random.randint(50000, 60000)
 
@@ -30,6 +31,15 @@ def _make_inst_and_monitor():
         my_monitor.stop()
         instr.close()
 
+
+@pytest.fixture(name="channel_instr")
+def _make_channel_instr():
+    instr = DummyChannelInstrument("MonitorDummy")
+    try:
+        yield instr
+    finally:
+        instr.close()
+
 # Test cases for the qcodes monitor
 
 def test_setup_teardown(request):
@@ -90,13 +100,12 @@ def test_connection(request):
     asyncio.set_event_loop(loop)
 
     async def async_test_connection():
-        websocket = await websockets.connect(f"ws://localhost:{monitor.WEBSOCKET_PORT}")
-        await websocket.close()
+        async with websockets.connect(f"ws://localhost:{monitor.WEBSOCKET_PORT}"):
+            pass
     loop.run_until_complete(async_test_connection())
 
     m.stop()
 
-
 def test_parameter(request, inst_and_monitor):
     """
     Test instrument updates
@@ -112,41 +121,83 @@ def test_parameter(request, inst_and_monitor):
     instr, my_monitor, monitor_parameters, param = inst_and_monitor
 
     async def async_test_monitor():
-        websocket = await websockets.connect(f"ws://localhost:{monitor.WEBSOCKET_PORT}")
+        async with websockets.connect(
+            f"ws://localhost:{monitor.WEBSOCKET_PORT}"
+        ) as websocket:
 
-        # Recieve data from monitor
-        data = await websocket.recv()
-        data = json.loads(data)
-        # Check fields
-        assert "ts" in data
-        assert "parameters" in data
-        # Check one instrument and "No Instrument" is being sent
-        assert len(data["parameters"]) == 2
-        assert data["parameters"][1]["instrument"] == "Unbound Parameter"
-        metadata = data["parameters"][0]
-        assert metadata["instrument"] == str(instr)
+            # Recieve data from monitor
+            data = await websocket.recv()
+            data = json.loads(data)
+            # Check fields
+            assert "ts" in data
+            assert "parameters" in data
+            # Check one instrument and "No Instrument" is being sent
+            assert len(data["parameters"]) == 2
+            assert data["parameters"][1]["instrument"] == "Unbound Parameter"
+            metadata = data["parameters"][0]
+            assert metadata["instrument"] == str(instr)
 
-        # Check parameter values
-        old_timestamps = {}
-        for local_param, mon in zip(monitor_parameters, metadata["parameters"]):
-            assert str(local_param.get_latest()) == mon["value"]
-            assert local_param.label == mon["name"]
-            old_timestamps[local_param.label] = float(mon["ts"])
-            local_param(random.random())
+            # Check parameter values
+            old_timestamps = {}
+            for local_param, mon in zip(monitor_parameters, metadata["parameters"]):
+                assert isinstance(local_param, Parameter)
+                assert str(local_param.get_latest()) == mon["value"]
+                assert local_param.label == mon["name"]
+                old_timestamps[local_param.label] = float(mon["ts"])
+                local_param(random.random())
 
-        # Check parameter updates
-        data_str = await websocket.recv()
-        data_str = await websocket.recv()
-        data = json.loads(data_str)
-        metadata = data["parameters"][0]
-        for local_param, mon in zip(monitor_parameters, metadata["parameters"]):
-            assert str(local_param.get_latest()) == mon["value"]
-            assert local_param.label == mon["name"]
-            assert float(mon["ts"]) > old_timestamps[local_param.label]
+            # Check parameter updates
+            data_str = await websocket.recv()
+            data_str = await websocket.recv()
+            data = json.loads(data_str)
+            metadata = data["parameters"][0]
+            for local_param, mon in zip(monitor_parameters, metadata["parameters"]):
+                assert str(local_param.get_latest()) == mon["value"]
+                assert isinstance(local_param, Parameter)
+                assert local_param.label == mon["name"]
+                assert float(mon["ts"]) > old_timestamps[local_param.label]
 
-        # Check unbound parameter
-        metadata = data["parameters"][1]["parameters"]
-        assert len(metadata) == 1
-        assert param.label == metadata[0]["name"]
+            # Check unbound parameter
+            metadata = data["parameters"][1]["parameters"]
+            assert len(metadata) == 1
+            assert param.label == metadata[0]["name"]
 
     loop.run_until_complete(async_test_monitor())
+
+
+@pytest.mark.parametrize("use_root_instrument", [True, False])
+def test_use_root_instrument(request, channel_instr, use_root_instrument):
+    """
+    Test instrument updates
+    """
+    loop = asyncio.new_event_loop()
+
+    def cleanup_loop():
+        loop.stop()
+        loop.close()
+
+    request.addfinalizer(cleanup_loop)
+
+    asyncio.set_event_loop(loop)
+
+    m = monitor.Monitor(
+        channel_instr.A.dummy_start,
+        channel_instr.B.dummy_start,
+        use_root_instrument=use_root_instrument,
+    )
+    request.addfinalizer(m.stop)
+
+    async def async_test_monitor(use_root_instrument):
+        async with websockets.connect(
+            f"ws://localhost:{monitor.WEBSOCKET_PORT}"
+        ) as websocket:
+
+            # Recieve data from monitor
+            data = await websocket.recv()
+            data = json.loads(data)
+            if use_root_instrument:
+                assert len(data["parameters"]) == 1
+            else:
+                assert len(data["parameters"]) == 2
+
+    loop.run_until_complete(async_test_monitor(use_root_instrument))
