@@ -1,5 +1,15 @@
-from typing import (Any, Dict, NamedTuple, NewType, Sequence, Tuple, TypeVar,
-                    Union, Optional)
+from typing import (
+    Any,
+    Dict,
+    Mapping,
+    NamedTuple,
+    NewType,
+    Optional,
+    Sequence,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
 from .helpers import deep_update
 
@@ -25,11 +35,11 @@ RunId = NewType('RunId', int)
 
 
 class Metadatable:
-    def __init__(self, metadata=None):
-        self.metadata = {}
+    def __init__(self, metadata: Optional[Mapping[str, Any]] = None):
+        self.metadata: Dict[str, Any] = {}
         self.load_metadata(metadata or {})
 
-    def load_metadata(self, metadata: dict) -> None:
+    def load_metadata(self, metadata: Mapping[str, Any]) -> None:
         """
         Load metadata into this classes metadata dictionary.
 
@@ -38,7 +48,7 @@ class Metadatable:
         """
         deep_update(self.metadata, metadata)
 
-    def snapshot(self, update: bool = False) -> Dict:
+    def snapshot(self, update: Optional[bool] = False) -> Dict[Any, Any]:
         """
         Decorate a snapshot dictionary with metadata.
         DO NOT override this method if you want metadata in the snapshot
@@ -59,8 +69,9 @@ class Metadatable:
         return snap
 
     def snapshot_base(
-            self, update: bool = False,
-            params_to_skip_update: Optional[Sequence[str]] = None) -> Dict:
+            self, update: Optional[bool] = False,
+            params_to_skip_update: Optional[Sequence[str]] = None
+    ) -> Dict[str, Any]:
         """
         Override this with the primary information for a subclass.
         """
@@ -82,16 +93,19 @@ def extract_param_values(snapshot: Snapshot) -> Dict[ParameterKey, Any]:
     instrument and parameter names onto parameter values.
     """
     parameters = {}
-    for param_name, param in snapshot['station']['parameters'].items():
+    snapshot = snapshot.get('station', snapshot)
+    for param_name, param in snapshot['parameters'].items():
         parameters[param_name] = param['value']
-    for instrument_name, instrument in snapshot['station']['instruments'].items():
-        for param_name, param in instrument['parameters'].items():
-            if 'value' in param:
-                parameters[instrument_name, param_name] = param['value']
+    if 'instruments' in snapshot:
+        for instrument_name, instrument in snapshot['instruments'].items():
+            for param_name, param in instrument['parameters'].items():
+                if 'value' in param:
+                    parameters[instrument_name, param_name] = param['value']
 
     return parameters
 
 
+
 def diff_param_values(left_snapshot: Snapshot,
                       right_snapshot: Snapshot
                       ) -> ParameterDiff:
@@ -99,8 +113,12 @@ def diff_param_values(left_snapshot: Snapshot,
     Given two snapshots, returns the differences between parameter values
     in each.
     """
-    left_params, right_params = map(extract_param_values, (left_snapshot, right_snapshot))
-    left_keys, right_keys = [set(params.keys()) for params in (left_params, right_params)]
+    left_params, right_params = map(
+        extract_param_values, (left_snapshot, right_snapshot)
+    )
+    left_keys, right_keys = (
+        set(params.keys()) for params in (left_params, right_params)
+    )
     common_keys = left_keys.intersection(right_keys)
 
     return ParameterDiff(
